<?php
declare(strict_types = 1);

namespace App\Http\Logics\Outpatient;

use App\Dictionary\Order\PayType;
use App\Dictionary\Order\SourceId;
use App\Dictionary\Order\Type;
use App\Dictionary\Patient\CardType;
use App\Exceptions\GeneralException;
use App\Http\Resources\Outpatient\Pending\PendingListsResource;
use App\Models\Order;
use App\Models\Patient;
use App\Services\HisHttp\Client;
use App\Utils\Traits\BuildCacheKeyName;
use App\Utils\Traits\Logger;
use App\Utils\Traits\MiniProgramAuth;
use Illuminate\Auth\AuthenticationException;
use Illuminate\Support\Facades\Redis;
use ReflectionException;
use Symfony\Component\HttpFoundation\Response;
use UnifyPayment\Cores\Exceptions\InvalidConfigException;
use UnifyPayment\Cores\Exceptions\RuntimeException;
use UnifyPayment\Cores\Struct\CreateOrder;
use UnifyPayment\Mock\CreateOrderHandler;
use UnifyPayment\Unify;

class PaymentLogic
{
    use Logger;
    use MiniProgramAuth;
    use BuildCacheKeyName;

    private Client $his_client;

    private Order $order_model;

    private Patient $patient_model;

    /**
     * PaymentLogic Construct
     * @throws AuthenticationException
     */
    public function __construct()
    {
        $this->authInitialize();
        $this->setChannel('outpatient');
        $this->his_client = app('HisHttpService');
        $this->order_model = new Order();
        $this->patient_model = new Patient();
    }

    /**
     * 支付
     * @param string $patient_number
     * @param string $serial_no
     * @param string $prescription_ids
     * @return array
     * @throws GeneralException
     */
    public function payment(string $patient_number, string $serial_no, string $prescription_ids): array
    {
        // 基础信息
        $patient_info = $this->getPatientInfo($this->open_id, $patient_number);
        $pending_info = $this->getPendingPrescriptionDetails($patient_info['patientId'], $serial_no, $prescription_ids);

        // 创建订单
        $order_type = Type::OUTPATIENT_PAYMENT;
        $pay_type = PayType::WECHAT_PAY;
        $order_id = $this->order_model->getOrderId($pay_type, 'M');
        $total_fee = (float)(string) $pending_info['prescriptionAmount'];

        $order = $this->createOrder($order_id, $pay_type, $total_fee, $order_type, $patient_info, $pending_info);

        // 申请支付
        $pay_data = $this->applyPayment($order_type, $order_id, $total_fee, $patient_info['patientNumber'], $patient_info['name']);

        // 去除无用数据
        unset($pay_data['merchantId'], $pay_data['merchantName'], $pay_data['channelId'], $pay_data['channelName']);

        return $pay_data;
    }

    /**
     * 获取患者信息
     * @param string $open_id
     * @param string $patient_number
     * @return mixed
     * @throws GeneralException
     */
    protected function getPatientInfo(string $open_id, string $patient_number): mixed
    {
        $info = $this->patient_model->getBindPatientInfoByPatientNumber($open_id, $patient_number);
        if (empty($info)) {
            throw new GeneralException('找不到患者信息，请重新再试！', Response::HTTP_BAD_REQUEST);
        }

        $response = $this->his_client->getPatientInfo($info['patient_number'], CardType::OUTPATIENT_NO, '');
        if (!isset($response['success']) || !$response['success']) {
            throw new GeneralException($response['msg'] ?? '找不到患者信息，请重新再试！', Response::HTTP_SERVICE_UNAVAILABLE);
        }

        // 添加Patient 表ID
        $patient_info = &$response['response'];
        $patient_info['id'] = $info['id'];

        $this->info('缴费患者信息', $patient_info);
        return $patient_info;
    }

    /**
     * 获取缴费处方详情
     * @param string $patient_id
     * @param string $serial_no
     * @param string $prescription_ids
     * @return array
     * @throws GeneralException
     */
    protected function getPendingPrescriptionDetails(string $patient_id, string $serial_no,  string $prescription_ids): array
    {
        // 缓存键值
        $pending_lists_cache_key = $this->getOutpatientPendingListsKey($this->open_id, $patient_id);
        $pending_details_cache_key = $this->getOutpatientPendingDetailsKey($this->open_id, $patient_id, $serial_no);

        // 获取缓存
        if (Redis::exists($pending_lists_cache_key)) {
            $response = Redis::get($pending_lists_cache_key);
            $response = json_decode($response, true);
        }

        // 如果缓存数据有问题
        if (empty($response)) {
            $response = $this->his_client->getPendingLists($patient_id);

            if (!isset($response['success']) || !$response['success']) {
                throw new GeneralException($response['msg'] ?? '暂无相关缴费记录！', Response::HTTP_SERVICE_UNAVAILABLE);
            }
        }

        // 获取具体缴费详情
        $lists = [];
        foreach ($response['response'] as $v) {
            if (
                str_contains($prescription_ids, $v['prescriptionId']) &&
                $serial_no == $v['visitNumber']
            ) {
                if (empty($lists)) {
                    $lists = $v;
                    $lists['prescription_ids'] = $v['prescriptionId'];
                    $lists['prescription_numbers'] = $v['nrescriptionNumber'];
                    // 去除无用信息
                    unset($lists['idCardNo'], $lists['singleAmount'], $lists['prescriptionId'], $lists['nrescriptionNumber']);
                } else {
                    // 拼接处方号
                    $lists['prescription_ids'] .= ','. $v['prescriptionId'];
                    $lists['prescription_numbers'] .= ','. $v['nrescriptionNumber'];
                }
            }
        }

        if (empty($lists) || array_diff(explode(',', $lists['prescription_ids']), explode(',', $prescription_ids))) {
            throw new GeneralException('查询不到待缴费处方，请重新再试！', Response::HTTP_SERVICE_UNAVAILABLE);
        }

        // 获取缓存
        if (Redis::exists($pending_details_cache_key)) {
            $response = Redis::get($pending_details_cache_key);
            $response = json_decode($response, true);
        }

        // 如果缓存数据有问题
        if (empty($response)) {
            $response = $this->his_client->getPendingDetails($prescription_ids, $serial_no, $lists['regId'] ?: '');

            if (!isset($response['success']) || !$response['success']) {
                throw new GeneralException($response['msg'] ?? '暂无相关缴费记录！', Response::HTTP_SERVICE_UNAVAILABLE);
            }
        }

        $lists['prescription_lists'] = $response['response'];

        $this->info('缴费处方信息', $lists);
        return $lists;
    }

    /**
     * 创建订单表
     * @param string $order_id
     * @param PayType $pay_type
     * @param float $total_fee
     * @param Type $order_type
     * @param array $patient_info
     * @param array $pending_info
     * @return mixed
     * @throws GeneralException
     */
    protected function createOrder(string $order_id, PayType $pay_type, float $total_fee, Type $order_type, array $patient_info, array $pending_info): mixed
    {
        // 挂号记录表
        $pay_record_data = [
            'relate_patient_id' => $patient_info['id'],
            'dept_id' => $pending_info['treatmentDepartment'],
            'dept_name' => $pending_info['departmentName'],
            'doctor_id' => $pending_info['doctorId'],
            'doctor_name' => $pending_info['doctorName'],
            'visit_date' => date('Y-m-d', strtotime($pending_info['visitDate'])),
            'total_amount' => $pending_info['prescriptionAmount'] * 100,
            'pre_settle_status' => 0,
            'extra_info' => json_encode($pending_info, JSON_UNESCAPED_UNICODE),
        ];

        $order = $this->order_model->createOrder(
            $patient_info['id'],
            $order_id,
            $pay_type,
            $total_fee * 100,
            0,
            $this->open_id,
            $patient_info['patientId'],
            $patient_info['name'],
            $order_type,
            SourceId::MINI_PROGRAM,
            $pay_record_data
        );
        if (empty($order)) {
            throw new GeneralException('创建缴费单失败，请重新再试！');
        }

        $this->info('创建订单,ID：'. $order->id);
        return $order;
    }

    /**
     * 申请支付
     * @param Type $order_type
     * @param string $order_id
     * @param float $total_fee
     * @param string $patient_id
     * @param string $patient_name
     * @return array
     * @throws GeneralException
     */
    protected function applyPayment(Type $order_type, string $order_id, float $total_fee, string $patient_id, string $patient_name): array
    {
        try {
            $order_obj = new CreateOrder(
                $order_type->label(),
                $order_id,
                (string) $total_fee,
                'tag001',
                $patient_id. '|'.  $patient_name,
                $order_type->unifyOrderType(),
                $this->open_id,
                config('custom.payment_notify_url')
            );

            $response = Unify::pay(config('unify'))->mini->jsapi($order_obj);
            $this->info('jsapi 支付参数', $response);

            if (!$response['success'] || empty($response['response'])) {
                throw new GeneralException('申请支付失败，请重新再试！', Response::HTTP_SERVICE_UNAVAILABLE);
            }

            return $response['response'];
        } catch (InvalidConfigException|RuntimeException|ReflectionException $e) {
            throw new GeneralException('申请支付失败，请重新再试！', Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }
}
